/*  This file is part of the KDE project
    Copyright (C) 2000 Simon Hausmann <hausmann@kde.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "kio_file.h"

#include <qdir.h>
#include <qfileinfo.h>

using namespace KIO;

#define MAX_IPC_SIZE (1024*32)

File::File()
    : SlaveBase( "file" )
{
}

File::~File()
{
}

void File::get( const KURL &url )
{
    QString path = url.path();

    QFileInfo info( path );

    if ( !info.exists() )
    {
        error( KIO::ERR_DOES_NOT_EXIST, path );
        return;
    }

    if ( info.isFile() )
    {
        QFile file( path );

        if ( !file.open( IO_ReadOnly ) )
        {
            error( KIO::ERR_CANNOT_OPEN_FOR_READING, path );
            return;
        }

        totalSize( info.size() );

        QByteArray buffer( MAX_IPC_SIZE );

        while ( !file.atEnd() )
        {
            int n = file.readBlock( buffer.data(), buffer.size() );

            if ( n == -1 )
            {
                error( KIO::ERR_COULD_NOT_READ, path );
                return;
            }

            data( buffer );
        }

        data( QByteArray() ); // ### why is that needed?
    }
    else if ( info.isDir() )
    {
        // ###
        error( KIO::ERR_IS_DIRECTORY, path );
        return;
    }

    finished();
}
