/*  This file is part of the KDE project
    Copyright (C) 1999 Simon Hausmann <hausmann@kde.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <kapp.h>
#include <kcookieserver.h>
#include <khtml_part.h>
#include <klibloader.h>
#include <ecma/kjs_proxy.h>
#include <kiconloader.h>

#include <signal.h>

#include "pics/file_broken.xpm"

// ### from ecma/kjs_proxy.cpp and khtml_factory.cpp
extern "C"
{
    KJSProxy *kjs_html_init( KHTMLPart * );
    void *init_libkhtml();
}

// ### from khtml_part.cpp
extern int kjs_lib_count;

#include "mainwindow.h"

int main( int argc, char **argv )
{
    signal( SIGCHLD, SIG_IGN );

    kjs_lib_count++; // ### hack, prevent khtmlpart from deleting the js klibrary object

    KApplication app( argc, argv, "konq-embed" );

    KCookieServer cookieEater;

    KLibLoader::self()->library( "kjs_html" )->registerSymbol( "kjs_html_init", (void *)&kjs_html_init );
    KLibLoader::self()->library( "libkhtml" )->registerSymbol( "init_libkhtml", (void *)&init_libkhtml );

    KIconLoader::self()->registerIcon( "file_broken", QPixmap( file_broken_xpm ) );

    QObject::connect( &app, SIGNAL( lastWindowClosed() ), &app, SLOT( quit() ) );

    bool openedMainWindow = false;
    for ( int i = 1; i < argc; i++ )
    {
       KURL u = QString::fromLatin1( QCString( argv[ i ] ) );
       if ( !u.isMalformed() && !u.isEmpty() )
       {
           MainWindow *toplevel = new MainWindow;
	   toplevel->resize( 800, 600 );
           toplevel->show();
           toplevel->openURL( u );
           openedMainWindow = true;
       }
    }

    if ( !openedMainWindow )
    {
        MainWindow *toplevel = new MainWindow;
	toplevel->resize( 800, 600 );
        toplevel->show();
        // toplevel->openURL( "http://www.kde.org/" );
    }

    return app.exec();
}
