/*  This file is part of the KDE project
    Copyright (C) 1999 Simon Hausmann <hausmann@kde.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

// !! The history code is (c) David Faure <faure@kde.org> !!

#include <klocale.h>

#include <qpopupmenu.h>
#include <qaction.h>
#include <qmenubar.h>
#include <qstatusbar.h>
#include <qtoolbar.h>
#include <qlineedit.h>
#include <qprogressbar.h>
#include <qlayout.h>
#include <qplatinumstyle.h>

#include <khtml_part.h>
#include <kio/job.h>
#include <kio/jobclasses.h>
#include <kmessagebox.h>

#include <assert.h>

#include "mainwindow.h"

#include "pics/stop.xpm"
#include "pics/lock.xpm"
#include "pics/unlock.xpm"
#include "pics/back.xpm"
#include "pics/forward.xpm"
#include "pics/gohome.xpm"
#include "pics/reload.xpm"
#include "pics/close.xpm"


MainWindow::MainWindow()
{
    setCaption( "NetRaider  (developer version)" );

    qApp->setStyle( new QPlatinumStyle );

    m_history.setAutoDelete( true );

    m_doc = new KHTMLPart( this, "khtmlpart" ); // ### frame name

    //connect( m_doc->browserExtension(), SIGNAL( loadingProgress( KIO::Job *job, int ) ),
     //        this, SLOT( slotLoadingProgress( KIO::Job *job, int ) ) );

    connect( m_doc->browserExtension(), SIGNAL( createNewWindow( const KURL &, const KParts::URLArgs & ) ),
             this, SLOT( slotCreateNewWindow( const KURL &, const KParts::URLArgs & ) ) );

    connect( m_doc, SIGNAL( setStatusBarText( const QString & ) ),
             this, SLOT( slotStatusBarText( const QString & ) ) );

    connect( m_doc->browserExtension(), SIGNAL( setLocationBarURL( const QString & ) ),
             this, SLOT( slotLocationBarText( const QString & ) ) );

    connect( m_doc->browserExtension(), SIGNAL( loadingProgress( int ) ),
            this, SLOT( slotLoadingProgress( int ) ) );

    connect( m_doc->browserExtension(), SIGNAL( openURLRequest( const KURL &, const KParts::URLArgs & ) ),
            this, SLOT( slotOpenURLRequest( const KURL &, const KParts::URLArgs & ) ) );

    //         this, SLOT( slotCreateNewWindow( const KURL &, const KParts::URLArgs & ) ) );

    connect( m_doc, SIGNAL( started( KIO::Job * ) ),
             this, SLOT( slotStarted( KIO::Job * ) ) );

    connect( m_doc, SIGNAL( completed() ),
             this, SLOT( slotCompleted() ) );

    connect( m_doc, SIGNAL( setWindowCaption( const QString & ) ),
             this, SLOT( slotSetWindowCaption( const QString & ) ) );

    //connect( m_doc, SIGNAL( loadingProgress( int ) ),
     //        this, SLOT( slotLoadingProgress( int ) ) );

    setCentralWidget( m_doc->widget() );
    setToolBarsMovable( true );

    initActions();
    initGUI();
}

MainWindow::~MainWindow()
{
}

void MainWindow::initActions()
{
    m_paNewWindow = new QAction( i18n( "&New Window" ), QIconSet(), i18n( "New Window" ), CTRL + Key_N, this, "newwindow");
    connect( m_paNewWindow, SIGNAL( activated() ), this, SLOT( slotNewWindow() ) );

    m_paDuplicateWindow = new QAction( i18n( "&Duplicate Window" ), QIconSet(), i18n( "Duplicate Window" ), CTRL + Key_D, this, "duplicatewindow");
    connect( m_paDuplicateWindow, SIGNAL( activated() ), this, SLOT( slotDuplicateWindow() ) );

    m_paClear = new QAction( i18n( "Clear" ), QIconSet( QPixmap( close_xpm ) ), i18n( "Clear Location" ), ALT + Key_X, this, "clear" );
    connect( m_paClear, SIGNAL( activated() ), this, SLOT( slotClear() ) );

    m_paClose = new QAction( i18n( "Close Window" ), QIconSet(), i18n( "Close Window" ), CTRL + Key_W, this, "close" );
    connect( m_paClose, SIGNAL( activated() ), this, SLOT( close() ) );

    m_paStop = new QAction( i18n( "Stop" ), QIconSet( QPixmap( stop_xpm ) ), i18n( "Stop Loading" ), Key_Escape, this, "stop" );
    connect( m_paStop, SIGNAL( activated() ), this, SLOT( slotStop() ) );
    m_paStop->setEnabled( false );

    m_paEnableJS = new QAction( i18n( "Enable Javascript" ), QIconSet(), i18n( "Enable Javascript" ), 0, this, "enablejs" );
    m_paEnableJS->setToggleAction( true );
    connect( m_paEnableJS, SIGNAL( toggled( bool ) ), this, SLOT( slotToggleJavaScript( bool ) ) );
    m_paEnableJS->setOn( true );

    m_paBack = new QAction( i18n( "Backward" ), QIconSet( QPixmap( back_xpm ) ), QString::null, ALT + Key_J, this, "back" );
    connect( m_paBack, SIGNAL( activated() ),
             this, SLOT( slotBack() ) );

    m_paForward = new QAction( i18n( "Forward" ), QIconSet( QPixmap( forward_xpm ) ), QString::null, ALT + Key_K, this, "forward" );
    connect( m_paForward, SIGNAL( activated() ),
             this, SLOT( slotForward() ) );

    m_paBack->setEnabled( false );
    m_paForward->setEnabled( false );

    m_paAddBookmark = new QAction( i18n( "&Add Bookmark" ), QIconSet(), QString::null, CTRL + Key_B, this );
    connect( m_paAddBookmark, SIGNAL( activated() ),
             this, SLOT( slotAddBookmark() ) );

    m_paHome = new QAction( i18n( "Home" ), QIconSet( QPixmap( gohome_xpm ) ), QString::null, 0, this );
    connect( m_paHome, SIGNAL( activated() ),
             this, SLOT( slotHome() ) );

    m_paReload = new QAction( i18n( "Reload" ), QIconSet( QPixmap( reload_xpm ) ), QString::null, Key_F5, this );
    connect( m_paReload, SIGNAL( activated() ),
             this, SLOT( slotReload() ) );

    m_paFind = new QAction( i18n( "Find in Document" ), QIconSet(), QString::null, 0, this );
    connect( m_paFind, SIGNAL( activated() ),
             this, SLOT( slotFind() ) );

    m_paCloseFind = new QAction( i18n( "Close Find" ), QIconSet( QPixmap( close_xpm ) ), QString::null, 0, this );
    connect( m_paCloseFind, SIGNAL( activated() ),
             this, SLOT( slotCloseFind() ) );

    m_paMenu = new QAction( i18n( "Menu" ), QIconSet( QPixmap( close_xpm ) ), QString::null, ALT + Key_M, this );
    connect( m_paMenu, SIGNAL( activated() ),
            this, SLOT( slotShowMenu() ) );
}

void MainWindow::initGUI()
{
    m_mainMenu = new QPopupMenu( this );
    
    //m_paFind->addTo( locationMenu );

    //menuBar->insertItem( i18n( "&Application" ), m_mainMenu );

    m_paNewWindow->addTo( m_mainMenu );

    m_paDuplicateWindow->addTo( m_mainMenu );

    m_mainMenu->insertSeparator();

    QPopupMenu *settingsMenu = new QPopupMenu( this );

    m_mainMenu->insertItem( i18n( "&Settings" ), settingsMenu );

    m_mainMenu->insertSeparator();


    m_bookmarksMenu = new QPopupMenu( this );
    connect( m_bookmarksMenu, SIGNAL( aboutToShow() ),
             this, SLOT( slotBookmarksMenuAboutToShow() ) );
    connect( m_bookmarksMenu, SIGNAL( activated( int ) ),
             this, SLOT( slotBookmarkActivated( int ) ) );
    m_mainMenu->insertItem( i18n( "&Bookmarks" ), m_bookmarksMenu );


    m_mainMenu->insertSeparator();

    m_paClose->addTo( m_mainMenu );

    m_paEnableJS->addTo( settingsMenu );


    QToolBar *tb = new QToolBar( i18n( "Toolbar" ), this, QMainWindow::Top, true, "toolbar" );

    tb->setHorizontalStretchable( true );

    m_paMenu->addTo( tb );

    m_paBack->addTo( tb );

    m_paForward->addTo( tb );

    m_paHome->addTo( tb );

    m_paReload->addTo( tb );

    m_paStop->addTo( tb );

    m_paClear->addTo( tb );

    m_locationLineEdit = new QLineEdit( m_doc->url().url(), tb, "locationlineedit" );
    connect( m_locationLineEdit, SIGNAL( returnPressed() ),
             this, SLOT( slotURLEntered() ) );
    m_locationLineEdit->setFocus();

    KAction *a = m_doc->actionCollection()->action( "security" );

    assert( a );

    a->assignIconSet( "unlock", QPixmap( unlock_xpm ) );
    a->assignIconSet( "lock", QPixmap( lock_xpm ) );
    a->setIcon( "unlock" );

    a->addTo( tb );


    //QStatusBar *sb = new QStatusBar( i18n( "Status" ), this, QMainWindow::Top, true, "statusbar" );
    //QStatusBar *statusBar = new QStatusBar( this );
    
    m_locationProgressBar = new QProgressBar( 100, tb, "locationprogressbar" );

    m_locationProgressBar->hide();

    statusBar()->show();

    statusBar()->setSizeGripEnabled( false );

    //m_searchBar = new QToolBar( "searchbar", this, QMainWindow::Top, true );

    //m_findInput = new QLineEdit( m_searchBar, "search" );
    //connect( m_findInput, SIGNAL( returnPressed() ),
    //         this, SLOT( slotFindNext() ) );

    //m_searchBar->setStretchableWidget( m_findInput );
}

void MainWindow::openURL( const KURL &url )
{
    slotOpenURLRequest( url, KParts::URLArgs() );
}

void MainWindow::slotOpenURLRequest( const KURL &url, const KParts::URLArgs &args )
{
    QString frameName = args.frameName.lower();
    if ( !frameName.isEmpty() && frameName == QString::fromLatin1( "_blank" ) )
    {
        slotCreateNewWindow( url, args );
     /*       MainWindow *newlevel = new MainWindow;
        newlevel->resize( 800, 600 );
        newlevel->show();

        newlevel->createHistoryEntry();
        newlevel->m_locationLineEdit->setText( url.prettyURL() );

        newlevel->m_doc->browserExtension()->setURLArgs( args );
        newlevel->m_doc->openURL( url );
        newlevel->updateHistoryEntry();
	*/
    } else {
        createHistoryEntry();
        updateHistoryEntry();
        m_paBack->setEnabled( canGoBack() );
        m_paForward->setEnabled( canGoForward() );
        m_locationLineEdit->setText( url.prettyURL() );
	m_doc->browserExtension()->setURLArgs( args );
	m_doc->openURL( url );
    }
}

void MainWindow::slotLoadingProgress( int percent )
{
    m_locationProgressBar->setProgress( percent );
    MainWindow *newlevel = new MainWindow;
    newlevel->resize( 800, 600 );
    newlevel->show();
}

void MainWindow::slotCreateNewWindow( const KURL &url, const KParts::URLArgs &args )
{
    MainWindow *newlevel = new MainWindow;
    newlevel->resize( 800, 600 );
    newlevel->show();

    newlevel->m_locationLineEdit->setText( url.prettyURL() );

    newlevel->m_doc->browserExtension()->setURLArgs( args );
    newlevel->m_doc->openURL( url );
}

void MainWindow::slotShowMenu()
{
    m_mainMenu->popup( QCursor().pos() ); 
}

void MainWindow::slotNewWindow()
{
    MainWindow *newlevel = new MainWindow;
    newlevel->resize( 800, 600 );
    newlevel->show();
}

void MainWindow::slotDuplicateWindow()
{
    MainWindow *newlevel = new MainWindow;
    newlevel->resize( 800, 600 );
    newlevel->show();
    newlevel->openURL( m_doc->url() );
}

void MainWindow::slotClear()
{
    m_locationProgressBar->hide();
    m_locationLineEdit->show();
    m_locationLineEdit->setText( "" );
    m_locationLineEdit->setFocus();
}

void MainWindow::slotStop()
{
    m_doc->closeURL();
    slotCompleted();
    m_locationLineEdit->setFocus();
}

void MainWindow::slotStarted( KIO::Job *job )
{
    m_paStop->setEnabled( true );

    if ( job )
        connect( job, SIGNAL( infoMessage( KIO::Job *, const QString & ) ),
                 this, SLOT( slotInfoMessage( KIO::Job *, const QString & ) ) );
    
    //connect( job, SIGNAL( loadingProgress( KIO::Job *, int ) ),
     //       this, SLOT( slotLoadingProgress( KIO::Job *, int ) ) );

    //m_locationLineEdit->hide();
    //m_locationProgressBar->show();

    //statusBar()->show();
}

void MainWindow::slotCompleted()
{
    m_paStop->setEnabled( false );
    updateHistoryEntry();

    m_paBack->setEnabled( canGoBack() );
    m_paForward->setEnabled( canGoForward() );

    //m_locationProgressBar->hide();
    //m_locationLineEdit->show();
}

void MainWindow::slotURLEntered()
{
    KURL u = m_locationLineEdit->text();

    //if ( u.isMalformed() && QRegExp( "^www" ).match( m_locationLineEdit->text() ) == 0 )
    //    u = KURL( m_locationLineEdit->text().prepend( "http://" ) );

    if ( QRegExp( "^find " ).match( m_locationLineEdit->text() ) == 0 )
        slotFindNext ( m_locationLineEdit->text().remove(0, 5) );
    //else if ( QRegExp( "^run " ).match( m_locationLineEdit->text() ) == 0 )
       // system( m_locationLineEdit->text().remove(0, 4) );
    else 
    {
        if ( u.isMalformed() ) 
            u = KURL( m_locationLineEdit->text().prepend( "http://" ) );

        if ( !u.isMalformed() )
        {
            //openURL( u );
	    m_doc->openURL( u );
        }
        else
            KMessageBox::error( this, i18n( "Malformed URL\n%1" ).arg( u.url() ) );
    }
}

void MainWindow::slotToggleJavaScript( bool toggle )
{
    m_doc->enableJScript( toggle );
}

void MainWindow::slotLocationBarText( const QString &url )
{
    m_locationLineEdit->setText( url );
}

void MainWindow::slotStatusBarText( const QString &text )
{
   /* if ( !text.isEmpty() ) {
	setCaption( text );
    } else {
        setCaption( m_currentTitle + " <NetRaider>" );
    }*/
    statusBar()->message( text );
}

void MainWindow::slotInfoMessage( KIO::Job *, const QString &text )
{
    statusBar()->message( text );
    //setCaption( text );
}

void MainWindow::slotSetWindowCaption( const QString &caption )
{
    m_currentTitle = caption;
    setCaption( caption + " > NetRaider <" );
}

void MainWindow::createHistoryEntry()
{
    HistoryEntry *current = m_history.current();

    if ( current )
    {
        m_history.at( m_history.count() - 1 );
        while ( m_history.current() != current )
	{
	    if ( !m_history.removeLast() )
	        assert(0);
	}
    }

    m_history.append( new HistoryEntry );
    assert( m_history.at() == (int) m_history.count() - 1 );
}

void MainWindow::updateHistoryEntry()
{
    //ASSERT( !m_blockHistory );

    HistoryEntry *current = m_history.current();

   // assert( current );

    if ( current == 0L) // empty history
    {
        current = new HistoryEntry;
	m_history.append( current );
    }

    if ( m_doc->browserExtension() )
    {
        QDataStream stream( current->m_buffer, IO_WriteOnly );

        m_doc->browserExtension()->saveState( stream );
    }
    current->m_url = m_doc->url();
}

void MainWindow::slotBack()
{
    slotStop();

    goHistory( -1 );
}

void MainWindow::slotForward()
{
    slotStop();

    goHistory( 1 );
}

void MainWindow::slotHome()
{
    openURL( "http://www.netraider.de/" );
}

void MainWindow::goHistory( int steps )
{
    int newPos = m_history.at() + steps;

    HistoryEntry *current = m_history.at( newPos );

    HistoryEntry h( *current );

    QDataStream stream( h.m_buffer, IO_ReadOnly );

    m_doc->browserExtension()->restoreState( stream );

    m_paBack->setEnabled( canGoBack() );
    m_paForward->setEnabled( canGoForward() );
}

void MainWindow::slotAddBookmark()
{
    Bookmark newBookmark;
    newBookmark.m_url = m_doc->url();
    newBookmark.m_title = m_currentTitle;
    m_bookmarks.append( newBookmark );
}

void MainWindow::slotBookmarksMenuAboutToShow()
{
    m_bookmarksMenu->clear();

    m_paAddBookmark->addTo( m_bookmarksMenu );
    m_bookmarksMenu->insertSeparator();

    QValueList<Bookmark>::ConstIterator it = m_bookmarks.begin();
    QValueList<Bookmark>::ConstIterator end = m_bookmarks.end();
    for (; it != end; ++it )
        m_bookmarksMenu->insertItem( (*it).m_title );
}

void MainWindow::slotBookmarkActivated( int id )
{
    int idx = m_bookmarksMenu->indexOf( id );

    if ( idx < 2 )
        return;

    Bookmark bm = m_bookmarks[ idx - 2 ];

    if ( !bm.m_url.isEmpty() && !bm.m_url.isMalformed() )
        openURL( bm.m_url );
}

void MainWindow::slotReload()
{
    KURL url = m_doc->url();
    if ( !url.isEmpty() && !url.isMalformed() )
    {
        KParts::URLArgs args;
        args.reload = true;
        //m_doc->browserExtension()->setURLArgs( args );
        //m_doc->openURL( url );
	openURL( url );
	//KMessageBox::error( this, i18n( "openURL ( %1 )" ).arg( url.url() ) );
    }
}

void MainWindow::slotFind()
{
    m_searchBar->show();
    m_doc->findTextBegin();
}

void MainWindow::slotCloseFind()
{
    m_searchBar->hide();
}

void MainWindow::slotFindNext( const QString &text )
{
    m_doc->findTextNext( text, true, false );
}

#include "mainwindow.moc"
