/*  This file is part of the KDE project
    Copyright (C) 1999 Simon Hausmann <hausmann@kde.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

// !! The history code is (c) David Faure <faure@kde.org> !!

#ifndef __mainwindow_h__
#define __mainwindow_h__

#include <qmainwindow.h>
#include <qcstring.h>
#include <qprogressbar.h>

#include <kurl.h>
#include <kparts/browserextension.h>

class QAction;
class KHTMLPart;
class QLineEdit;
class QProgressBar;

namespace KIO
{
    class Job;
};

class MainWindow : public QMainWindow
{
    Q_OBJECT
public:
    MainWindow();
    virtual ~MainWindow();

    KHTMLPart *doc() const { return m_doc; }

    void openURL( const KURL &url );

private slots:
    void slotShowMenu();
	
    void slotLoadingProgress( int percent );

    void slotClear();
	
    void slotCreateNewWindow( const KURL &url, const KParts::URLArgs &args );

    void slotNewWindow();

    void slotDuplicateWindow();

    void slotStatusBarText( const QString &text );
    
    void slotLocationBarText( const QString &text );
    
    void slotOpenURLRequest( const KURL &url, const KParts::URLArgs &args );
    void slotStop();

    void slotStarted( KIO::Job *job );
    void slotCompleted();

    void slotURLEntered();

    void slotToggleJavaScript( bool toggle );

    void slotInfoMessage( KIO::Job *, const QString &text );

    void slotSetWindowCaption( const QString &caption );

    void slotBack();

    void slotForward();

    void slotHome();

    void slotAddBookmark();

    void slotBookmarksMenuAboutToShow();

    void slotBookmarkActivated( int id );

    void slotReload();

    void slotFind();

    void slotFindNext( const QString &text );

    void slotCloseFind();

private:
    void initActions();

    void initGUI();

    void createHistoryEntry();

    void updateHistoryEntry();

    bool canGoBack() { return m_history.at() > 0; }

    bool canGoForward() { return m_history.at() != (int)(m_history.count() - 1); }

    void goHistory( int steps );

    struct HistoryEntry
    {
        KURL m_url;
        QByteArray m_buffer;
    };

    QList<HistoryEntry> m_history;

    QAction *m_paMenu;
    QAction *m_paClear;
    QAction *m_paNewWindow;
    QAction *m_paDuplicateWindow;
    QAction *m_paClose;
    QAction *m_paStop;
    QAction *m_paBack;
    QAction *m_paForward;
    QAction *m_paHome;
    QAction *m_paReload;
    QAction *m_paFind;
    QAction *m_paCloseFind;

    QToolBar *m_searchBar;
    QLineEdit *m_findInput;

    QAction *m_paEnableJS;

    QAction *m_paAddBookmark;

    KHTMLPart *m_doc;

    QLineEdit *m_locationLineEdit;

    QProgressBar *m_locationProgressBar;
    
    QString m_currentTitle;

    QPopupMenu *m_bookmarksMenu;

    QPopupMenu *m_mainMenu;

    struct Bookmark
    {
        KURL m_url;
        QString m_title;
    };

    QValueList<Bookmark> m_bookmarks;
};

#endif
